# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Package visvis.utils

This package contains a few modules and packages that are stand-alone
and maybe useful outside visvis. Some utilities are used inside visvis
(such as ssdf), others extend the functionality of visvis or have some
relation to visualization.

The intended way to import utilities is: "from visvis.utils import utilname".

The currrent utilities are:

  * ssdf - simple structured data format.
  * pypoints - Representnig points, pointsets, anisotropic arrays and quaternions.
  * graph - Representing points/nodes that are connected by edges.
  * cropper - small app based on visvis controls to crop 3D data

"""
